classNameKey = 'className'
methodNameKey = 'methodName'
preHookKey = 'preHook'
postHookKey = 'postHook'

AGENT_VERSION = '1.0.0'
NEXTAPM_HOST = 'NEXTAPM_HOST'
NEXTAPM_LICENSE_KEY = 'NEXTAPM_LICENSE_KEY'
NEXTAPM_PROJECT_ID = 'NEXTAPM_PROJECT_ID'
DEFAULT_COLLECTOR = 'https://data.nextapm.dev'




import threading

threadLocal = threading.local()

def setCurTxn(txn):
    setattr(threadLocal, 'NextApmCurTxn', txn)

def clearCurTxn():
    setCurTxn(None)

def getCurTxn():
    return getattr(threadLocal, 'NextApmCurTxn', None)

def isTxnActive():
    txn = getCurTxn()
    return txn is not None





import nextapm.lib.logger as logger
from .modules import modulesInfo
from importlib import import_module
from ..util import isCallable
from ..constants import postHookKey, preHookKey, methodNameKey, classNameKey

def callHook(methodInfo, key, args, kwargs):
    try:
        hook = methodInfo.get(key, '')
        if not hook:
            return
            
        hook(args, kwargs)
    except Exception as err:
        logger.error('[callHook]', err)


def wrap(original, methodInfo):
    def wrapper(*args, **kwargs):
        callHook(methodInfo, preHookKey, args, kwargs)
        try:
            res = original(*args, **kwargs)
            return res
        except Exception as exc:
            raise exc
        finally:
            callHook(methodInfo, postHookKey, args, kwargs)

    wrapper.__name__ = original.__name__
    return wrapper


def instrumentModule(modName, actModule):
    if not modName:
        return

    if hasattr(actModule, 'nextapm_instrumented'):
        return 

    if modName in modulesInfo.keys():
        methodsInfo = modulesInfo.get(modName)
        for eachMethodInfo in methodsInfo:
            instrumentMethod(modName, actModule, eachMethodInfo)

        setattr(actModule, 'nextapm_instrumented', True)
        logger.info(modName+' instrumented')


def instrumentMethod(modName, actModule, methodInfo):
    parent = actModule

    if classNameKey in methodInfo:
        className = methodInfo.get(classNameKey)
        if hasattr(actModule, className):
            parent = getattr(actModule, className)


    methodName = methodInfo.get(methodNameKey, '')
    if hasattr(parent, methodName):
        original = getattr(parent, methodName)
        if not isCallable(original):
            return
        
        wrapper = wrap(original, methodInfo)
        setattr(parent,  methodName, wrapper)


def start():
    logger.info('starting instrumentation')
    for eachModule in modulesInfo:
        try:
            actModule = import_module(eachModule)
            instrumentModule(eachModule, actModule)
        except Exception as e:
            logger.error('unable to instument '+ eachModule, e)




import os
import time
import nextapm.lib.logger as logger
from .constants import NEXTAPM_LICENSE_KEY, NEXTAPM_PROJECT_ID

try:
    from collections.abc import Callable
except ImportError:
    from collections import Callable

def currentMillis():
    return int(round(time.time() * 1000))

def isCallable(fn):
    return isinstance(fn, Callable)

def validateConfig():
    licenseKey = os.getenv(NEXTAPM_LICENSE_KEY, '')
    if not licenseKey:
        logger.error('Configure license key in NEXTAPM_LICENSE_KEY environment variable')
        return False
    
    projectId = os.getenv(NEXTAPM_PROJECT_ID, '') 
    if not projectId:
        logger.error('Configure project id in NEXTAPM_PROJECT_ID environment variable')
        return False

    return True



from ..data.context import setCurTxn, getCurTxn
from ..data.txn import Transaction
from ..constants import classNameKey, methodNameKey, preHookKey, postHookKey

def captureTxn(args, kwargs):
    txn = Transaction()
    setCurTxn(txn)

def endTxn(args, kwargs):
    txn = getCurTxn()
    if txn is None or args is None:
        return

    if len(args) <= 0:
        return

    obj = args[0]
    if obj is None:
        return
    
    if hasattr(obj,'path'):
        txn.setUrl(getattr(obj,'path'))
    
    if hasattr(obj,'command'):
        txn.setMethod(getattr(obj, 'command'))

    txn.end()


def extractStatusCode(args, kwargs):
    txn = getCurTxn()
    if txn is None or args is None:
        return

    if len(args) <= 1:
        return
    
    txn.setStatus(args[1])


modulesInfo = {
  'http.server': [
      {
          classNameKey: 'BaseHTTPRequestHandler',
          methodNameKey: 'handle_one_request',
          preHookKey: captureTxn,
          postHookKey: endTxn
      },
      {
          classNameKey: 'BaseHTTPRequestHandler',
          methodNameKey: 'send_response',
          postHookKey: extractStatusCode
      }
  ]
}




from nextapm.lib.util import currentMillis
from nextapm.lib.data.transport import sendMetrics
import nextapm.lib.logger as logger

class Transaction:

    def __init__(self, info={}):
        self.url = info.get('path', '')
        self.method = info.get('command', '')
        self.startTime = currentMillis()
        self.endTime = None
        self.rt = 0
        self.completed = False
        self.status = 200
        self.exceptionsInfo = {}

    def end(self):
        try:
            self.endTime = currentMillis()
            self.rt = self.endTime - self.startTime
            self.completed = True
            sendMetrics(self)
        except Exception as e:
            logger.error('[Transaction][end]', e)

    def print(self):
        print({
          'url': self.url,
          'method': self.method,
          'start': self.startTime,
          'rt': self.rt,
          'status': self.status,
        })

    def getAsJson(self):
        return {
          'url': self.url,
          'method': self.method,
          'start': self.startTime,
          'status': self.status,
          'rt': self.rt,
        }

    def isCompleted(self):
        return self.completed

    def setUrl(self, url):
        if type(url) is str:
            self.url = url

    def setMethod(self, method):
        if type(method) is str:
            self.method = method

    def setStatus(self, code):
        if type(code) is int:
            self.status = code



import nextapm.lib.logger as logger
from nextapm.lib.util import validateConfig
from nextapm.lib.constants import *
import os
import json
import requests

def sendData(data):
    if not validateConfig():
        return
  
    if data is None or type(data) is not dict:
        logger.error('[sendData] empty data passed', data)
        return

    try:
        logger.debug('[sendData] Data', data)
        host = os.getenv(NEXTAPM_HOST, DEFAULT_COLLECTOR)
        licenseKey = os.getenv(NEXTAPM_LICENSE_KEY)
        projectId = os.getenv(NEXTAPM_PROJECT_ID)
        queryParams = 'licenseKey='+licenseKey+'&projectId='+projectId
        url = host+'/api/data?'+queryParams
        payload = json.dumps(data)
        headers = {'content-type': 'application/json'}
        response = requests.post(url, data=payload, headers=headers)
        logger.info('collector response '+ json.dumps(response.json()))
    except Exception as exc:
        logger.error('req error ', exc)

    

def sendMetrics(txn):
    try:
        logger.debug('[sendMetric]')
        if not validateConfig():
            return False

        if txn is None:
            logger.error('[sendMetric] invalid txn')
            return False

        if not txn.isCompleted():
            logger.warn('[sendMetric] txn not completed')
            return False

        sendData({
            'info': {
              'agent_version': AGENT_VERSION
            },
            'txn': txn.getAsJson()
        })
    except Exception as e:
        logger.error('[sendMetric]', e)

    return False



import os

LEVELS = ['DEBUG', 'INFO', 'WARN', 'ERROR']

levelStr = os.getenv('NEXTAPM_LOG_LEVEL', 3)
configuredLevel = int(levelStr)

def log(level, msg, obj):
    if configuredLevel <= level:
        level = LEVELS[configuredLevel]
        logMsg = '[NextApm][Agent]['+level+'] ' + str(msg)
        print(logMsg, obj)


def debug(msg, data={}):
    log(0, msg, data)

def info(msg, data={}):
    log(1, msg, data)

def warn(msg, data={}):
    log(2, msg, data)

def error(msg, e=None):
    log(3, msg, e)



import setuptools

with open("README.md", "r") as fh:
	long_description = fh.read()

requirements = ['requests']

setuptools.setup(name='nextapm',
      version='1.0.0',
      description='Monitor vercel serverless python api',
      long_description=long_description,
      long_description_content_type="text/markdown",
      url='https://nextapm.dev',
      author='Chan Anan',
      author_email='chan@nextapm.dev',
      license='MIT',
      zip_safe=False,
      packages=setuptools.find_packages(),
      python_requires='>=3.6',
      install_requires=requirements)
